@@ -33,6 +33,7 @@
 #include <InternalErr.h>
 
 using namespace std;
+using namespace libdap;
 
 class ClientParams {
 private:
@@ -82,11 +83,9 @@ public:
         The \c string_as_char_array parameter tells the library to treat DAP
         String variables as character arrays. 
         
-        @note The default value is true; in some cases it might be better to
-        treat String variables as single characters since that's how the netcdf
-        data handler we wrote treats them. Thus, if the remote data source
-        is likely a netCDF file, users might want to set this parameter to 
-        false.
+        @note The default value is false. Set this to true when reading data 
+	from HDF4 files or other data sources which use the DAP String data
+	type for character arrays.
         
         @return The value of the parameter. */
     bool get_string_as_char_array() const { return d_string_as_char_array; }
