@@ -44,7 +44,7 @@ struct dim_proj_info {
     int stride;
     
     dim_proj_info() : start(0), stop(0), stride(0) {}
-    dim_proj_info(string clause) {
+    dim_proj_info(string clause) : start(0), stop(0), stride(1){
         // Extract the indices and store the values.
         // First, replace syntax with spaces...
         string::size_type separator = 0;
@@ -69,14 +69,12 @@ struct dim_proj_info {
                 }
                 // Found start, stop but no stride
                 start = i1;
-                stride = 1;
                 stop = i2;
                 return;
             }
             // Found start but no stride or stop
             start = i1;
             stop = i1;
-            stride = 1;
             return;
         }
     }
