@@ -29,10 +29,10 @@
 //  Reza 5/18/99
 // 
 
-// $Id: Dncx.cc 11906 2005-08-08 19:51:43Z root $ 
+// $Id: Dncx.cc 18782 2008-05-23 16:39:07Z jimg $ 
 
 #include "Dncx.h"
-#include <string.h>
+#include <cstring>
 #include <limits.h>
 /* alias poorly named limits.h macros */
 #define  SHORT_MAX  SHRT_MAX
