@@ -40,12 +40,12 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: Dputget.cc 15133 2006-11-22 21:02:06Z jimg $"};
+static char rcsid[] not_used ={"$Id: Dputget.cc 16556 2007-05-23 21:13:45Z jimg $"};
 
 #include <sstream>
 
-#include "Str.h"
-#include "Error.h"
+#include <Str.h>
+#include <Error.h>
 
 //#define DODS_DEBUG
 #include "debug.h"
