@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: ldim.c 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: ldim.c 14988 2006-11-01 22:21:09Z jimg $ */
 
 #include "lnc.h"
 #include <stdlib.h>
@@ -321,9 +321,15 @@ lnc_def_dim(int ncid, const char *name, size_t size, int *dimidp)
 	if(status != NC_NOERR)
 		return status;
 
-		/* cast needed for braindead systems with signed size_t */
-	if((unsigned long) size > X_INT_MAX) /* Backward compat */
-		return NC_EINVAL;
+	if ((ncp->flags & NC_64BIT_OFFSET) && sizeof(off_t) > 4) {
+	    /* CDF2 format and LFS */
+	    if(size > X_UINT_MAX - 3) /* "- 3" handles rounded-up size */
+		return NC_EDIMSIZE;
+	} else {
+	    /* CDF1 format */
+	    if(size > X_INT_MAX - 3)
+		return NC_EDIMSIZE;
+	}
 
 	if(size == NC_UNLIMITED)
 	{
