@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *	See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* "$Id: lncx.h 11906 2005-08-08 19:51:43Z root $" */
+/* "$Id: lncx.h 14988 2006-11-01 22:21:09Z jimg $" */
 
 #ifndef _NCX_H_
 #define _NCX_H_
@@ -22,7 +22,7 @@
  * 
  */
 
-#include "ncconfig.h" /* output of 'configure' */
+#include <config.h> /* output of 'configure' */
 #include "lrnd.h"
 #include <stddef.h> /* size_t */
 #include <errno.h>
@@ -34,6 +34,13 @@
 #define CRAYFLOAT 1 /* CRAY Floating point */
 #endif
 
+
+#if defined(DLL_NETCDF) /* define when library is a DLL */
+#include <io.h>
+#define lseek _lseeki64
+#define off_t __int64
+#endif	/* defined(DLL_NETCDF) */
+
 /*
  * The integer return code for the conversion routines
  * is 0 (ENOERR) when no error occured, or NC_ERANGE as appropriate
@@ -56,15 +63,20 @@
 #define X_SIZEOF_CHAR		1
 #define X_SIZEOF_SHORT		2
 #define X_SIZEOF_INT		4	/* xdr_int */
-
+#if 0
+#define X_SIZEOF_LONG		8 */	/* xdr_long_long */
+#endif
 #define X_SIZEOF_FLOAT		4
 #define X_SIZEOF_DOUBLE		8
 
 /*
- * For now, netcdf is limited to 32 bit offsets and sizes,
+ * For now, netcdf is limited to 32 bit sizes,
+ * If compiled with support for "large files", then
+ * netcdf will use a 64 bit off_t and it can then write a file
+ * using 64 bit offsets. 
  *  see also X_SIZE_MAX, X_OFF_MAX below
  */
-#define X_SIZEOF_OFF_T		X_SIZEOF_INT
+#define X_SIZEOF_OFF_T		(sizeof(off_t))
 #define X_SIZEOF_SIZE_T		X_SIZEOF_INT
 
 /*
@@ -82,8 +94,7 @@
 #define X_INT_MIN	(-2147483647-1)
 #define X_INT_MAX	2147483647
 #define X_UINT_MAX	4294967295U
-
-#define X_FLOAT_MAX	3.40282347e+38f
+#define X_FLOAT_MAX	3.402823466e+38f
 #define X_FLOAT_MIN	(-X_FLOAT_MAX)
 #define X_FLT_MAX	X_FLOAT_MAX	/* alias compatible with limits.h */
 #if CRAYFLOAT
@@ -96,7 +107,7 @@
 #define X_DOUBLE_MIN	(-X_DOUBLE_MAX)
 #define X_DBL_MAX	X_DOUBLE_MAX	/* alias compatible with limits.h */
 
-#define X_SIZE_MAX	X_INT_MAX	/* N.B., just uses the signed range */
+#define X_SIZE_MAX	X_UINT_MAX
 #define X_OFF_MAX	X_INT_MAX
 
 
@@ -187,6 +198,39 @@ typedef signed char schar;
  *
  */
 
+#if 0
+extern int
+ncx_get_schar_schar(const void *xp, schar *ip);
+extern int
+ncx_get_schar_uchar(const void *xp, uchar *ip);
+extern int
+ncx_get_schar_short(const void *xp, short *ip);
+extern int
+ncx_get_schar_int(const void *xp, int *ip);
+extern int
+ncx_get_schar_long(const void *xp, long *ip);
+extern int
+ncx_get_schar_float(const void *xp, float *ip);
+extern int
+ncx_get_schar_double(const void *xp, double *ip);
+
+extern int
+ncx_put_schar_schar(void *xp, const schar *ip);
+extern int
+ncx_put_schar_uchar(void *xp, const uchar *ip);
+extern int
+ncx_put_schar_short(void *xp, const short *ip);
+extern int
+ncx_put_schar_int(void *xp, const int *ip);
+extern int
+ncx_put_schar_long(void *xp, const long *ip);
+extern int
+ncx_put_schar_float(void *xp, const float *ip);
+extern int
+ncx_put_schar_double(void *xp, const double *ip);
+#endif
+ 
+
 extern int
 ncx_get_short_schar(const void *xp, schar *ip);
 extern int
@@ -322,14 +366,14 @@ extern int
 ncx_get_size_t(const void **xpp, size_t *ulp);
 /* ncx_get_int_off_t */
 extern int
-ncx_get_off_t(const void **xpp, off_t *lp);
+ncx_get_off_t(const void **xpp, off_t *lp, size_t sizeof_off_t);
 
 /* ncx_put_int_size_t */
 extern int
 ncx_put_size_t(void **xpp, const size_t *ulp);
 /* ncx_put_int_off_t */
 extern int
-ncx_put_off_t(void **xpp, const off_t *lp);
+ncx_put_off_t(void **xpp, const off_t *lp, size_t sizeof_off_t);
 
 
 /*
