@@ -2,9 +2,9 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *	See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: lposixio.c 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: lposixio.c 14988 2006-11-01 22:21:09Z jimg $ */
 
-#include "ncconfig.h"
+#include <config.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -51,6 +51,22 @@
 #undef X_ALIGN
 #endif
 
+/* These are needed on mingw to get a dll to compile. They really
+ * should be provided in sys/stats.h, but what the heck. Let's not be
+ * too picky! */
+#ifndef S_IRGRP
+#define S_IRGRP   0000040
+#endif
+#ifndef S_IROTH
+#define S_IROTH   0000004
+#endif
+#ifndef S_IWGRP
+#define S_IWGRP   0000020
+#endif
+#ifndef S_IWOTH
+#define S_IWOTH   0000002
+#endif
+
 /*
  * Define the following for debugging.
  */
@@ -134,9 +150,51 @@ fgrow(const int fd, const off_t len)
 	return ENOERR;
 }
 
+
+/*
+ * Sortof like ftruncate, except won't make the file shorter.  Differs
+ * from fgrow by only writing one byte at designated seek position, if
+ * needed.
+ */
+static int
+fgrow2(const int fd, const off_t len)
+{
+	struct stat sb;
+	if (fstat(fd, &sb) < 0)
+		return errno;
+	if (len <= sb.st_size)
+		return ENOERR;
+	{
+	    const char dumb = 0;
+	    /* we don't use ftruncate() due to problem with FAT32 file systems */
+	    /* cache current position */
+	    const off_t pos = lseek(fd, 0, SEEK_CUR);
+	    if(pos < 0)
+		return errno;
+	    if (lseek(fd, len-1, SEEK_SET) < 0)
+		return errno;
+	    if(write(fd, &dumb, sizeof(dumb)) < 0)
+		return errno;
+	    if (lseek(fd, pos, SEEK_SET) < 0)
+		return errno;
+	}
+	return ENOERR;
+}
 /* End OS */
 /* Begin px */
 
+/* The px_ functions are for posix systems, when NC_SHARE is not in
+   effect. */
+
+/* Write out a "page" of data to the file. The size of the page
+   (i.e. the extent) varies.
+
+   nciop - pointer to the file metadata.
+   offset - where in the file should this page be written.
+   extent - how many bytes should be written.
+   vp - pointer to the data to write.
+   posp - pointer to current position in file, updated after write.
+*/
 static int
 px_pgout(ncio *const nciop, 
 	off_t const offset,  const size_t extent,
@@ -165,7 +223,15 @@ px_pgout(ncio *const nciop,
 	return ENOERR;
 }
 
+/* Read in a page of data. 
 
+   nciop - a pointer to the ncio struct for this file.
+   offset - byte offset in file where read starts.
+   extent - the size of the page that will be read.
+   vp - a pointer to where the data will end up.
+   nreadp - returned number of bytes actually read (may be less than extent).
+   posp - pointer to current position in file, updated after read.
+*/
 static int
 px_pgin(ncio *const nciop,
 	off_t const offset, const size_t extent,
@@ -207,7 +273,21 @@ px_pgin(ncio *const nciop,
 	return ENOERR;
 }
 
+/* This struct is for POSIX systems, with NC_SHARE not in effect. If
+   NC_SHARE is used, see ncio_spx.
 
+   blksz - block size for reads and writes to file.
+   pos - current read/write position in file.
+   bf_offset - file offset corresponding to start of memory buffer
+   bf_extent - number of bytes in I/O request
+   bf_cnt - number of bytes available in buffer
+   bf_base - pointer to beginning of buffer.
+   bf_rflags - buffer region flags (defined in lncio.h) tell the lock
+   status, read/write permissions, and modification status of regions
+   of data in the buffer.
+   bf_refcount - buffer reference count.
+   slave - used in moves.
+*/
 typedef struct ncio_px {
 	size_t blksz;
 	off_t pos;
@@ -224,6 +304,20 @@ typedef struct ncio_px {
 
 
 /*ARGSUSED*/
+/* This function indicates the file region starting at offset may be
+   released.
+
+   This is for POSIX, without NC_SHARE.  If called with RGN_MODIFIED
+   flag, sets the modified flag in pxp->bf_rflags and decrements the
+   reference count.
+
+   pxp - pointer to posix non-share ncio_px struct.
+
+   offset - file offset for beginning of to region to be
+   released.
+
+   rflags - only RGN_MODIFIED is relevent to this function, others ignored
+*/
 static int
 px_rel(ncio_px *const pxp, off_t offset, int rflags)
 {
@@ -241,6 +335,22 @@ px_rel(ncio_px *const pxp, off_t offset, int rflags)
 	return ENOERR;
 }
 
+/* This function indicates the file region starting at offset may be
+   released.  Each read or write to the file is bracketed by a call to
+   the "get" region function and a call to the "rel" region function.
+   If you only read from the memory region, release it with a flag of
+   0, if you modify the region, release it with a flag of
+   RGN_MODIFIED.
+
+   For POSIX system, without NC_SHARE, this becomes the rel function
+   pointed to by the ncio rel function pointer. It mearly checks for
+   file write permission, then calls px_rel to do everything.
+
+   nciop - pointer to ncio struct.
+   offset - num bytes from beginning of buffer to region to be
+   released.
+   rflags - only RGN_MODIFIED is relevent to this function, others ignored
+*/
 static int
 ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
 {
@@ -252,6 +362,41 @@ ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
 	return px_rel(pxp, offset, rflags);
 }
 
+/* POSIX get. This will "make a region available." Since we're using
+   buffered IO, this means that if needed, we'll fetch a new page from
+   the file, otherwise, just return a pointer to what's in memory
+   already. 
+
+   nciop - pointer to ncio struct, containing file info.
+   pxp - pointer to ncio_px struct, which contains special metadate
+   for posix files without NC_SHARE.
+   offset - start byte of region to get.
+   extent - how many bytes to read.
+   rflags - One of the RGN_* flags defined in lncio.h.
+   vpp - pointer to pointer that will recieve data.
+
+   NOTES: 
+
+   * For blkoffset round offset down to the nearest pxp->blksz. This
+   provides the offset (in bytes) to the beginning of the block that
+   holds the current offset.
+
+   * diff tells how far into the current block we are.
+
+   * For blkextent round up to the number of bytes at the beginning of
+   the next block, after the one that holds our current position, plus
+   whatever extra (i.e. the extent) that we are about to grab.
+
+   * The blkextent can't be more than twice the pxp->blksz. That's
+   because the pxp->blksize is the sizehint, and in ncio_px_init2 the
+   buffer (pointed to by pxp->bf-base) is allocated with 2 *
+   *sizehintp. This is checked (unneccesarily) more than once in
+   asserts.
+
+   * If this is called on a newly opened file, pxp->bf_offset will be
+   OFF_NONE and we'll jump to label pgin to immediately read in a
+   page. 
+*/
 static int
 px_get(ncio *const nciop, ncio_px *const pxp,
 		off_t offset, size_t extent,
@@ -311,7 +456,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 	/* else */
 
 	if(pxp->bf_extent > pxp->blksz
-		 && blkoffset == pxp->bf_offset + pxp->blksz)
+		 && blkoffset == pxp->bf_offset + (off_t)pxp->blksz)
 	{
 		/* hit in upper half */
 		if(blkextent == pxp->blksz)
@@ -366,7 +511,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 	}
 	/* else */
 
-	if(blkoffset == pxp->bf_offset - pxp->blksz)
+	if(blkoffset == pxp->bf_offset - (off_t)pxp->blksz)
 	{
 		/* wants the page below */
 		void *const middle =
@@ -459,6 +604,26 @@ done:
 	return ENOERR;
 }
 
+/* Request that the region (offset, extent) be made available through
+   *vpp.
+
+   This function converts a file region specified by an offset and
+   extent to a memory pointer. The region may be locked until the
+   corresponding call to rel().
+
+   For POSIX systems, without NC_SHARE. This function gets a page of
+   size extent? 
+
+   This is a wrapper for the function px_get, which does all the heavy
+   lifting.
+
+   nciop - pointer to ncio struct for this file.
+   offset - offset (from beginning of file?) to the data we want to
+   read.
+   extent - the number of bytes to read from the file.
+   rflags - One of the RGN_* flags defined in lncio.h.
+   vpp - handle to point at data when it's been read.
+*/
 static int
 ncio_px_get(ncio *const nciop, 
 		off_t offset, size_t extent,
@@ -546,6 +711,20 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 	return status;
 }
 
+/* Like memmove(), safely move possibly overlapping data.
+
+   Copy one region to another without making anything available to
+   higher layers. May be just implemented in terms of get() and rel(),
+   or may be tricky to be efficient. Only used in by lnc_enddef()
+   after redefinition.
+
+   nciop - pointer to ncio struct with file info.
+   to - src for move?
+   from - dest for move?
+   nbytes - number of bytes to move.
+   rflags - One of the RGN_* flags defined in lncio.h. The only
+   reasonable flag value is RGN_NOLOCK.
+*/
 static int
 ncio_px_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
@@ -650,6 +829,9 @@ fprintf(stderr, "\tncio_px_move small\n");
 }
 
 
+/* Flush any buffers to disk. May be a no-op on if I/O is unbuffered.
+   This function is used when NC_SHARE is NOT used.
+*/
 static int
 ncio_px_sync(ncio *const nciop)
 {
@@ -677,6 +859,9 @@ ncio_px_sync(ncio *const nciop)
 	return status;
 }
 
+/* Internal function called at close to
+   free up anything hanging off pvt.
+*/
 static void
 ncio_px_free(void *const pvt)
 {
@@ -707,6 +892,25 @@ ncio_px_free(void *const pvt)
 }
 
 
+/* This is the second half of the ncio initialization. This is called
+   after the file has actually been opened.
+
+   The most important thing that happens is the allocation of a block
+   of memory at pxp->bf_base. This is going to be twice the size of
+   the chunksizehint (rounded up to the nearest sizeof(double)) passed
+   in from lnc__create or lnc__open. The rounded chunksizehint (passed
+   in here in sizehintp) is going to be stored as pxp->blksize.
+
+   According to our "contract" we are not allowed to ask for an extent
+   larger than this chunksize/sizehint/blksize from the ncio get
+   function.
+
+   nciop - pointer to the ncio struct
+   sizehintp - pointer to a size hint that will be rounded up and
+   passed back to the caller.
+   isNew - true if this is being called from ncio_create for a new
+   file.
+*/
 static int
 ncio_px_init2(ncio *const nciop, size_t *sizehintp, int isNew)
 {
@@ -737,6 +941,12 @@ ncio_px_init2(ncio *const nciop, size_t *sizehintp, int isNew)
 }
 
 
+/* This is the first of a two-part initialization of the ncio struct.
+   Here the rel, get, move, sync, and free function pointers are set
+   to their POSIX non-NC_SHARE functions (ncio_px_*).
+
+   The ncio_px struct is also partially initialized.
+*/
 static void
 ncio_px_init(ncio *const nciop)
 {
@@ -761,6 +971,9 @@ ncio_px_init(ncio *const nciop)
 
 /* Begin spx */
 
+/* This is the struct that gets hung of ncio->pvt(?) when the NC_SHARE
+   flag is used.
+*/
 typedef struct ncio_spx {
 	off_t pos;
 	/* buffer */
@@ -772,6 +985,22 @@ typedef struct ncio_spx {
 
 
 /*ARGSUSED*/
+/* This function releases the region specified by offset.
+
+   For POSIX system, with NC_SHARE, this becomes the rel function
+   pointed to by the ncio rel function pointer. It mearly checks for
+   file write permission, then calls px_rel to do everything.
+
+   nciop - pointer to ncio struct.
+
+   offset - beginning of region.
+
+   rflags - One of the RGN_* flags defined in lncio.h. If set to
+   RGN_MODIFIED it means that the data in this region were modified,
+   and it needs to be written out to the disk immediately (since we
+   are not buffering with NC_SHARE on).
+
+*/
 static int
 ncio_spx_rel(ncio *const nciop, off_t offset, int rflags)
 {
@@ -802,6 +1031,22 @@ ncio_spx_rel(ncio *const nciop, off_t offset, int rflags)
 }
 
 
+/* Request that the region (offset, extent) be made available through
+   *vpp.
+
+   This function converts a file region specified by an offset and
+   extent to a memory pointer. The region may be locked until the
+   corresponding call to rel().
+
+   For POSIX systems, with NC_SHARE.
+
+   nciop - pointer to ncio struct for this file.
+   offset - offset (from beginning of file?) to the data we want to
+   read.
+   extent - the number of bytes we want.
+   rflags - One of the RGN_* flags defined in lncio.h. May be RGN_NOLOCK.
+   vpp - handle to point at data when it's been read.
+*/
 static int
 ncio_spx_get(ncio *const nciop,
 		off_t offset, size_t extent,
@@ -951,6 +1196,17 @@ fprintf(stderr, "strategy %ld at %ld to %ld\n",
 }
 #endif
 
+/* Copy one region to another without making anything available to
+   higher layers. May be just implemented in terms of get() and rel(),
+   or may be tricky to be efficient.  Only used in by lnc_enddef()
+   after redefinition.
+
+   nciop - pointer to ncio struct for this file.
+   to - dest for move?
+   from - src for move?
+   nbytes - number of bytes to move.
+   rflags - One of the RGN_* flags defined in lncio.h.
+*/
 static int
 ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
@@ -1001,6 +1257,8 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 
 
 /*ARGSUSED*/
+/* Flush any buffers to disk. May be a no-op on if I/O is unbuffered.
+*/
 static int
 ncio_spx_sync(ncio *const nciop)
 {
@@ -1026,6 +1284,14 @@ ncio_spx_free(void *const pvt)
 }
 
 
+/* This does the second half of the ncio_spx struct initialization for
+   POSIX systems, with NC_SHARE on. 
+
+   nciop - pointer to ncio struct for this file. File has been opened.
+   sizehintp - pointer to a size which will be rounded up to the
+   nearest 8-byt boundary and then used as the max size "chunk" (or
+   page) to read from the file. 
+*/
 static int
 ncio_spx_init2(ncio *const nciop, const size_t *const sizehintp)
 {
@@ -1049,6 +1315,10 @@ ncio_spx_init2(ncio *const nciop, const size_t *const sizehintp)
 }
 
 
+/* First half of init for ncio_spx struct, setting the rel, get, move,
+   snyc, and free function pointers to the NC_SHARE versions of these
+   functions (i.e. the ncio_spx_* functions).
+*/
 static void
 ncio_spx_init(ncio *const nciop)
 {
@@ -1070,6 +1340,11 @@ ncio_spx_init(ncio *const nciop)
 
 /* */
 
+/* This will call whatever free function is attached to the free
+   function pointer in ncio. It's called from ncio_close, and from
+   ncio_open and ncio_create when an error occurs that the file
+   metadata must be freed.
+*/
 static void
 ncio_free(ncio *nciop)
 {
@@ -1083,6 +1358,10 @@ ncio_free(ncio *nciop)
 }
 
 
+/* Create a new ncio struct to hold info about the file. This will
+   create and init the ncio_px or ncio_spx struct (the latter if
+   NC_SHARE is used.)
+*/
 static ncio *
 ncio_new(const char *path, int ioflags)
 {
@@ -1135,6 +1414,21 @@ ncio_new(const char *path, int ioflags)
 #define NC_DEFAULT_CREAT_MODE 0666
 #endif
 
+/* Create a file, and the ncio struct to go with it. This funtion is
+   only called from lnc__create_mp.
+
+   path - path of file to create.
+   ioflags - flags from lnc_create
+   initialsz - From the netcdf man page: "The argument
+   Iinitialsize sets the initial size of the file at creation time."
+   igeto - 
+   igetsz - 
+   sizehintp - this eventually goes into pxp->blksz and is the size of
+   a page of data for buffered reads and writes.
+   nciopp - pointer to a pointer that will get location of newly
+   created and inited ncio struct.
+   igetvpp - pointer to pointer which will get the location of ?
+*/
 int
 ncio_create(const char *path, int ioflags,
 	size_t initialsz,
@@ -1230,6 +1524,49 @@ unwind_new:
 }
 
 
+/* This function opens the data file. It is only called from nc.c,
+   from lnc__open_mp and lnc_delete_mp.
+
+   path - path of data file.
+
+   ioflags - flags passed into lnc_open.
+   
+   igeto - looks like this function can do an initial page get, and
+   igeto is going to be the offset for that. But it appears to be
+   unused 
+
+   igetsz - the size in bytes of initial page get (a.k.a. extent). Not
+   ever used in the library.
+   
+   sizehintp - pointer to sizehint parameter from lnc__open or
+   lnc__create. This is used to set pxp->blksz. 
+   
+   Here's what the man page has to say:
+
+   "The argument referenced by chunksize controls a space versus time
+   tradeoff, memory allocated in the netcdf library versus number of
+   system calls.
+
+   Because of internal requirements, the value may not be set to
+   exactly the value requested. The actual value chosen is returned by reference.
+
+   Using the value NC_SIZEHINT_DEFAULT causes the library to choose a
+   default. How the system choses the default depends on the
+   system. On many systems, the "preferred I/O block size" is
+   available from the stat() system call, struct stat member
+   st_blksize. If this is available it is used. Lacking that, twice
+   the system pagesize is used. Lacking a call to discover the system
+   pagesize, we just set default chunksize to 8192.
+
+   The chunksize is a property of a given open netcdf descriptor ncid,
+   it is not a persistent property of the netcdf dataset."
+
+   nciopp - pointer to pointer that will get address of newly created
+   and inited ncio struct.
+
+   igetvpp - handle to pass back pointer to data from inital page
+   read, if this were ever used, which it isn't.
+*/
 int
 ncio_open(const char *path,
 	int ioflags,
@@ -1302,7 +1639,60 @@ unwind_new:
 	return status;
 }
 
+/* 
+ * Get file size in bytes.
+ */
+int
+ncio_filesize(ncio *nciop, off_t *filesizep)
+{
+    struct stat sb;
 
+    assert(nciop != NULL);
+    if (fstat(nciop->fd, &sb) < 0)
+	return errno;
+    *filesizep = sb.st_size;
+    return ENOERR;
+}
+
+/*
+ * Sync any changes to disk, then truncate or extend file so its size
+ * is length.  This is only intended to be called before close, if the
+ * file is open for writing and the actual size does not match the
+ * calculated size, perhaps as the result of having been previously
+ * written in NOFILL mode.
+ */
+int
+ncio_pad_length(ncio *nciop, off_t length)
+{
+	int status = ENOERR;
+
+	if(nciop == NULL)
+		return EINVAL;
+
+	if(!fIsSet(nciop->ioflags, NC_WRITE))
+	        return EPERM; /* attempt to write readonly file */
+
+	status = nciop->sync(nciop);
+	if(status != ENOERR)
+	        return status;
+
+ 	status = fgrow2(nciop->fd, length);
+ 	if(status != ENOERR)
+	        return status;
+	return ENOERR;
+}
+
+
+/* Write out any dirty buffers to disk and
+   ensure that next read will get data from disk.
+
+   Sync any changes, then close the open file associated with the ncio
+   struct, and free its memory.
+   
+   nciop - pointer to ncio to close.
+
+   doUnlink - if true, unlink file
+*/
 int 
 ncio_close(ncio *nciop, int doUnlink)
 {
