@@ -53,16 +53,12 @@ public:
     ~NCConnectTest() {}
 
     void setUp() {
-        cp = new ClientParams("http://localhost/");
+        cp = new ClientParams("[string_as_char_array=true]http://localhost/");
         ncc = new NCConnect("http://localhost/", cp);
     }
 
     void tearDown() {
 	   delete ncc; ncc = 0;
-#if 0
-           // NCConnect now deletes this.
-           delete cp; cp = 0;
-#endif
     }
 
     CPPUNIT_TEST_SUITE( NCConnectTest );
@@ -354,10 +350,7 @@ main( int argc, char* argv[] )
     CppUnit::TextTestRunner runner;
     runner.addTest( CppUnit::TestFactoryRegistry::getRegistry().makeTest() );
 
-    runner.run();
+    bool wasSuccessful = runner.run( "", false ) ;
 
-    return 0;
+    return wasSuccessful ? 0 : 1;
 }
-
-
-
